﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using System.Windows.Forms;
using MetroFramework.Forms;
using System.Net.NetworkInformation;
using System.Threading;

namespace Microscope_Control_Application
{

    public partial class ControlForm : MetroForm
    {
        private int captureCounter = 0;
        private bool bandpassFilter;
        private int pixelRangeMin, pixelRangeMax;
        private int steps;
        private decimal wavelength;
        private decimal medianRefractiveIndex;
        private decimal pixelsPerMicron, stepSize;
        private decimal offsetSize;

        #region Support Functions

        private void LoadSettings(string settingsName)
        {
            savedName.Text = settingsName;
            XmlDocument settingsFile = new XmlDocument();
            settingsFile.Load("settings.xml");
            foreach (XmlNode setting in settingsFile.DocumentElement)
            {
                if (setting.Attributes[0].Value == settingsName)
                {
                    bandpassFilter = bool.Parse(setting["bandpass-filter"].InnerText);
                    incPixelMin.Enabled = incPixelMax.Enabled = bandpassFilter;
                    pixelRangeMin = int.Parse(setting["pixel-min"].InnerText);
                    pixelRangeMax = int.Parse(setting["pixel-max"].InnerText);
                    steps = int.Parse(setting["steps"].InnerText);
                    wavelength = decimal.Parse(setting["wavelength"].InnerText);
                    medianRefractiveIndex = decimal.Parse(setting["mri"].InnerText);
                    pixelsPerMicron = decimal.Parse(setting["ppm"].InnerText);
                    bool stepSizeDefault = bool.Parse(setting["step-size"].Attributes[0].Value);
                    togDefaultStep.Checked = stepSizeDefault;
                    incSS.Enabled = !stepSizeDefault;
                    stepSize = stepSizeDefault ? 1 / pixelsPerMicron : decimal.Parse(setting["step-size"].InnerText);
                    offsetSize = decimal.Parse(setting["offset"].InnerText);
                    break;
                }
            }

            BpFilterToggle.Checked = bandpassFilter;
            incPixelMin.Value = pixelRangeMin;
            incPixelMax.Value = pixelRangeMax;
            incWavelength.Value = wavelength;
            incMRI.Value = medianRefractiveIndex;
            incPPM.Value = pixelsPerMicron;
            incSS.Value = stepSize;
            incOffset.Value = offsetSize;

        }
 
        #endregion

        public ControlForm()
        {
            using (Ping pingPi = new Ping())
            {
                PingReply response = pingPi.Send("192.168.0.10");
                if (response.Status.ToString() != "Success")
                {
                    MessageBox.Show("Microscope is not connected, unable to boot");
                    Environment.Exit(0);
                }
            }

            InitializeComponent();
            this.WindowState = FormWindowState.Maximized;
            LoadSettings("Default");

            editPanel.Location = savedName.Location;

            Commands.RunPiCommand("python scripts/streaming.py");
            Thread.Sleep(1000);
            livestreamView.playlist.add("tcp/h264://192.168.0.10:8000/");
            livestreamView.playlist.play();

        }
    

        #region Setting Adjustment Functions
        private void BpFilterToggle_CheckedChanged(object sender, EventArgs e)
        {
            bandpassFilter = BpFilterToggle.Checked;
            incPixelMin.Enabled = incPixelMax.Enabled = BpFilterToggle.Checked;
        }

        private void incPixelMin_ValueChanged(object sender, EventArgs e)
        {
            pixelRangeMin = (int)incPixelMin.Value;
            incPixelMax.Minimum = pixelRangeMin + 1;
        }

        private void incSteps_ValueChanged(object sender, EventArgs e)
        {
            steps = (int)incSteps.Value;
        }

        private void incWavelength_ValueChanged(object sender, EventArgs e)
        {
            wavelength = incWavelength.Value;
        }

        private void incMRI_ValueChanged(object sender, EventArgs e)
        {
            medianRefractiveIndex = incMRI.Value;
        }

        private void incPPM_ValueChanged(object sender, EventArgs e)
        {
            pixelsPerMicron = incPPM.Value;
        }

        private void togDefaultStep_CheckedChanged(object sender, EventArgs e)
        {
            incSS.Enabled = !togDefaultStep.Checked;
            if (togDefaultStep.Checked)
            {
                stepSize = incSS.Value = 1 / pixelsPerMicron;
            }
        }

        private void incOffset_ValueChanged(object sender, EventArgs e)
        {
            offsetSize = incOffset.Value;
        }

        private void incPixelMax_ValueChanged(object sender, EventArgs e)
        {
            pixelRangeMax = (int)incPixelMax.Value;
            incPixelMin.Maximum = pixelRangeMax - 1;
        }

        private void savedName_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            changeSaveName.Text = savedName.Text;
            savedName.Visible = false;
            editPanel.Visible = true;
        }

        private void saveNewName_Click(object sender, EventArgs e)
        {
            savedName.Text = changeSaveName.Text;
            editPanel.Visible = false;
            savedName.Visible = true;
        }

        private void ControlCenter_Load(object sender, EventArgs e)
        {

        }

        private void captureBackgroundImage_Click(object sender, EventArgs e)
        {
            //CaptureForm captureForm = new CaptureForm(this, false);
            //this.Enabled = false;
            //captureForm.Show();

            System.IO.File.Create("Z:/handshaking/capturebackground.txt");
        }

        private void captureBackgroundSample_Click(object sender, EventArgs e)
        {
            //CaptureForm backgroundCapture = new CaptureForm(this, true);
            //this.Enabled = false;
            //backgroundCapture.Show();
            //this.captureCounter += 1;

            System.IO.File.Create("Z:/handshaking/capturebackground/capturebackground" + this.captureCounter + ".txt");

        }

        private void settingsPanel_Paint(object sender, PaintEventArgs e)
        {

        }

        private void ControlCenter_FormClosed(object sender, FormClosedEventArgs e)
        {
            //System.IO.File.Create("Z:/handshaking/stopstreaming.txt");
        }

        private void captureImage_Click(object sender, EventArgs e)
        {
            this.captureCounter += 1;
            System.IO.File.Create("Z:/handshaking/captureimage/captureimage" + this.captureCounter + ".txt");
            System.Threading.Thread.Sleep(3000);
            System.IO.File.Move("Z:/temp/temp.png", "images/" + DateTime.Now.ToString("hmmss") + ".png");
        }

        private void saveAsButton_Click(object sender, EventArgs e)
        {
            MetroForm dialogBox = new MetroForm();
            dialogBox.Show();
        }

        #endregion
    }
}





































